/*
    @author: doreshnikov
    Daniil Oreshnikov, ITMO University
*/

#include <bits/stdc++.h>

using namespace std;

typedef long long ll;
typedef unsigned int uint;
typedef pair<int, int> pii;

template<typename T1, typename T2>
istream &operator>>(istream &in, pair<T1, T2> &p) {
    return in >> p.first >> p.second;
}

template<typename T>
istream &operator>>(istream &in, vector<T> &v) {
    for (auto &item : v)
        in >> item;
    return in;
}

void mcin() {}

template<typename T, typename ...Tail>
void mcin(T &&arg, Tail &&...args) {
    cin >> arg;
    mcin(args...);
}
#define in(type, ...) type __VA_ARGS__; mcin(__VA_ARGS__);

#define all(c) c.begin(), c.end()

struct edge {
    int from, to, a, b;
};

typedef vector<vector<int>> graph;

inline vector<int> dfs1(graph const &g) {
    vector<int> out;
    vector<bool> vis(g.size(), false);

    function<void(int)> walk = [&](int v) {
        vis[v] = true;
        for (int u : g[v]) {
            if (!vis[u]) walk(u);
        }
        out.push_back(v);
    };

    for (int v = 0; v < g.size(); v++) {
        if (!vis[v]) walk(v);
    }
    reverse(all(out));
    return out;
}

inline vector<int> dfs2(graph const &g, vector<int> const &order) {
    vector<int> comp(g.size(), -1);

    function<void(int, int)> walk = [&](int v, int c) {
        comp[v] = c;
        for (int u : g[v]) {
            if (comp[u] == -1) walk(u, c);
        }
    };

    int c = 0;
    for (int v : order) {
        if (comp[v] == -1) walk(v, c++);
    }
    return comp;
}

int main() {

    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);

    in(int, n)
    int m = 2 * n;
    vector<edge> edges(m);
    vector<vector<int>> d_in(n);
    int max_b = 0;

    for (int i = 0; i < m; i++) {
        int from = i / 2;
        in(int, to, a, b);
        edges[i] = edge{from, --to, a, b};
        d_in[to].push_back(i);
        max_b = max(max_b, b + 1);
    }

    int total = m + max_b;
    // m x edges, max_b x le, m x !edges, max_b x !le
    graph g(2 * total), gt(2 * total);
    auto inv = [&](int x) {
        return x < total ? x + total : x - total;
    };
    auto add = [&](int x, int y) {
        int nx = inv(x), ny = inv(y);
        g[x].push_back(y);
        g[ny].push_back(nx);
        gt[y].push_back(x);
        gt[nx].push_back(ny);
    };
    for (int i = 0; i < n; i++) {
        add(2 * i, inv(2 * i + 1));
        add(inv(2 * i), 2 * i + 1);
        add(d_in[i][0], inv(d_in[i][1]));
        add(inv(d_in[i][0]), d_in[i][1]);
    }
    for (int i = 0; i < m; i++) {
        add(i, inv(m + edges[i].a - 1));
        add(i, m + edges[i].b);
    }
    for (int i = 0; i < max_b - 1; i++) {
        add(m + i, m + i + 1);
    }

    auto order = dfs1(g);
    auto comp = dfs2(gt, order);
    vector<int> select(total);
    int w = 0;
    for (int i = 0; i < total; i++) {
        int ni = inv(i);
        if (comp[i] == comp[ni]) {
            cout << -1 << '\n';
            return 0;
        }
        select[i] = comp[i] > comp[ni];
    }
    while (!select[m + w]) w++;

    cout << w << '\n';
    for (int i = 0; i < m; i++) {
        if (select[i]) {
            cout << i + 1 << ' ';
        }
    }
    cout << '\n';

}