using namespace std;

#include <iostream>
#include <unordered_set>
#include <unordered_map>
//#include "testlib.h"
#include "jngen.h"

typedef long long ll;

const ll MAX_M = 200000;
const int MAX_W = 1000'000'000;

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);
    parseArgs(argc, argv);

    int max_n = getOpt("n");
    int n = 1 + rnd.wnext(max_n, 10);

    double cutProb = getOpt<double>("c", 0);

    int gw = getOpt<int>("gw", 0);
    bool makeTree = getOpt<bool>("tree", false);

    Graph g;
    if (makeTree) {
        Tree t = Tree::randomPrim(n, gw);
        g = t;
    } else {
        int max_m = (int) min((ll) n * (n - 1) / 2, (ll) getOpt("m", MAX_M));
        int m = 1 + rnd.wnext(max_m, 10);
        g = Graph::random(n, m).connected(m >= n - 1).g();
    }

    const int max_d = getOpt("d", 1000);

    int s = rnd.next(0, n - 1);
    vector<int> a(n);
    a[s] = rnd.next(1, max_d);
    int cf = a[s];
    set<int> next_e;
    set<int> vnodes;
    vnodes.insert(s);
    const auto geg = g.edges();
    vector <vector<int>> gne(n);
    for (int i = 0; i < geg.size(); ++i) {
        gne[geg[i].first].push_back(i);
        gne[geg[i].second].push_back(i);
    }
    vector<int> w(geg.size());
    for (int i: gne[s]) {
        next_e.insert(i);
    }

    while (!next_e.empty()) {
        const auto e = next(next_e.begin(), rnd.next((int) next_e.size()));
        int ei = *e;
        next_e.erase(e);
        int v = geg[ei].first;
        int u = geg[ei].second;
        if (rnd.next() < cutProb) {
            w[ei] = min(MAX_W, cf + 1 + rnd.wnext(max_d, -20));
        } else {
            w[ei] = max(1, cf - rnd.wnext(max_d, -20));
        }
        if (vnodes.find(geg[ei].first) != vnodes.end()) {
            if (vnodes.find(geg[ei].second) != vnodes.end()) {
                continue;
            }
            swap(u, v);
        }

        a[v] = 1 + rnd.wnext(max_d, 10);
        cf = min(MAX_W, 1 + rnd.wnext(a[v], 10));
        vnodes.insert(v);
        for (int i: gne[v]) {
            if (vnodes.find(geg[i].first) == vnodes.end() && geg[i].second == v ||
                vnodes.find(geg[i].second) == vnodes.end() && geg[i].first == u) {
                next_e.insert(i);
            }
        }
    }

    for (int i = 0; i < n; ++i) {
        if (a[i] == 0) {
            a[i] = rnd.next(1, cf);
        }
    }
    for (int i = 0; i < geg.size(); ++i) {
        if (w[i] == 0) {
            cf = min(MAX_W, cf + rnd.next(1, max_d));
            w[i] = cf;
        }
    }

    printf("%d %d %d\n", g.n(), (int) g.edges().size(), s + 1);
    for (int i = 0; i < n; ++i) {
        if (i > 0) {
            printf(" ");
        }
        printf("%d", a[i]);
    }
    printf("\n");

    for (int i = 0; i < geg.size(); ++i) {
        auto e = geg[i];
        printf("%d %d %d\n", e.first + 1, e.second + 1, w[i]);
    }
    return 0;
}
