/*
    @author: doreshnikov
    Daniil Oreshnikov, ITMO University
*/

#include <iostream>
#include <vector>
#include <string>
#include <tuple>

using namespace std;

typedef long long ll;
typedef unsigned int uint;
typedef pair<int, int> pii;

template<typename T1, typename T2>
istream &operator>>(istream &in, pair<T1, T2> &p) {
    return in >> p.first >> p.second;
}

template<typename T>
istream &operator>>(istream &in, vector<T> &v) {
    for (auto &item : v)
        in >> item;
    return in;
}

void mcin() {}

template<typename T, typename ...Tail>
void mcin(T &&arg, Tail &&...args) {
    cin >> arg;
    mcin(args...);
}
#define in(type, ...) type __VA_ARGS__; mcin(__VA_ARGS__);

#define all(c) c.begin(), c.end()

int const INF = 1e7;

template<typename T>
struct indexed_pair {
    T first{}, second{};

    indexed_pair() : first(), second() {};

    indexed_pair(T const &f, T const &s) : first(f), second(s) {};

    T &operator[](int idx) {
        return idx ? second : first;
    }
};

typedef tuple<int, int, int> backtrack;
typedef indexed_pair<int> item;
typedef indexed_pair<backtrack> trace;

int main() {

    in(int, n, m)
    vector<vector<int>> table(n, vector<int>(m));
    for (int i = 0; i < n; i++) {
        in(string, s)
        for (int j = 0; j < m; j++) {
            table[i][j] = s[j] - '0';
        }
    }

    vector<vector<item >> dp(n, vector<item>(m, item(INF, INF)));
    vector<vector<trace >> back(n, vector<trace>(m));
    if (table[0][0]) {
        dp[0][0] = item(0, 2);
    } else {
        dp[0][0] = item(1, 1);
    }
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            if (i == 0 && j == 0) continue;
            for (int r = 0; r < 2; r++) {
                int c = table[i][j] ^ r ^ 1;
                if (i > 0) {
                    int r_prev = table[i - 1][j] ^ c ^ 1;
                    int value = dp[i - 1][j][r_prev] + r;
                    if (dp[i][j][r] > value) {
                        dp[i][j][r] = value;
                        back[i][j][r] = backtrack{i - 1, j, r_prev};
                    }
                }
                if (j > 0) {
                    int value = dp[i][j - 1][r] + c;
                    if (dp[i][j][r] > value) {
                        dp[i][j][r] = value;
                        back[i][j][r] = backtrack{i, j - 1, r};
                    }
                }
            }
        }
    }

    int result = min(dp[n - 1][m - 1][0], dp[n - 1][m - 1][1]);
    if (result >= INF) {
        cout << -1 << '\n';
        return 0;
    }

    int i = n - 1, j = m - 1, r = 0;
    if (dp[i][j][1] < dp[i][j][0]) r = 1;
    vector<int> ri, ci;
    while (true) {
        int c = table[i][j] ^ r ^ 1;
        if (r && (ri.size() == 0 || ri.back() != i)) ri.push_back(i);
        if (c && (ci.size() == 0 || ci.back() != j)) ci.push_back(j);
        if (i == 0 && j == 0) break;
        tie(i, j, r) = back[i][j][r];
    }

    cout << ri.size() << ' ' << ci.size() << '\n';
    for (int rid : ri) cout << rid + 1 << ' ';
    cout << '\n';
    for (int cid : ci) cout << cid + 1 << ' ';
    cout << '\n';

}